#include "Scene.hh"

#include "renderables/Renderable.hh"

using namespace glow;
using namespace glow::viewer;

void Scene::add(const SharedRenderable& r) { mRenderables.push_back(r); }

aabb Scene::computeAabb() const
{
    aabb bounds;
    for (auto const& r : mRenderables)
        if (!r->isEmpty())
            bounds.add(r->computeAabb());
    return bounds;
}

bool Scene::has_3d_renderables() const { return !computeAabb().isEmpty(); }

bool Scene::shouldBeCleared() const
{
    if (clearAccumulation)
        return true;
    for (auto const& r : mRenderables)
        if (r->isDirty())
            return true;
    return false;
}

Scene::BoundingInfo Scene::getBoundingInfo() const
{
    auto aabb = computeAabb();
    if (aabb.isEmpty())
    {
        // Centered unit aabb for algorithm sanity down the line
        aabb.min = tg::pos3(-.5f);
        aabb.max = tg::pos3(.5f);
    }

    auto const size = aabb.size();
    return BoundingInfo{length(size), aabb.center(), tg::aabb3(aabb.min, aabb.max)};
}
